{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

Autor       : Pasha A. Poskraykoff
Module      : PrinterEx.pas
Description : More Info About Printers in your system and Jobs
Creation    : 11.09.98
Mail me     : pap@savva.ru
Version     : 2.0

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
unit PrinterEx;

interface

uses
  Windows, Forms, SysUtils, Classes, Winspool;

type
  EInvalidIndex = class(Exception);

  TPrinterInfo = record
    ServerName: String; PrinterName    : String; ShareName     : String;
    PortName  : String; DriverName     : String; Comment       : String;
    Location  : String; SepFile        : String; PrintProcessor: String;
    Datatype  : String; Parameters     : String; Attributes    : DWORD ;
    Priority  : DWORD ; DefaultPriority: DWORD ; StartTime     : DWORD ;
    UntilTime : DWORD ; Status         : DWORD ; Jobs          : DWORD ;
    AveragePPM: DWORD ;
  end;
  PPrinterInfo = ^TPrinterInfo;

  TJobInfo = record
    JobId       : DWORD      ; PrinterName   : String; MachineName : String;
    UserName    : String     ; Document      : String; NotifyName  : String;
    Datatype    : String     ; PrintProcessor: String; Parameters  : String;
    DriverName  : String     ; dStatus       : String; dwStatus    : DWORD ;
    Priority    : DWORD      ; Position      : DWORD ; StartTime   : DWORD ;
    UntilTime   : DWORD      ; TotalPages    : DWORD ; Size        : DWORD ;
    Submitted   : TSystemTime; Time          : DWORD ; PagesPrinted: DWORD ;
  end;
  PJobInfo = ^TJobInfo;

  TPrinterEx = class(TComponent)
  private
    { Private declarations }
    FPrinters: TList;
    FJobs    : TList;

    function  GetPrinter    (Index: Integer): TPrinterInfo;
    function  GetPrinterProp(Index: Integer): Boolean;
    function  GetPrintersCount              : Integer;

    procedure GetJobs(ID: THandle);
    function  GetJobInfo(Index: Integer): TJobInfo;
    function  GetJobsCount              : Integer;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy;                    override;

    procedure UpdatePrinters;
    function  UpdateJobs(Index: Integer): Boolean;

    property Printer        [Index: Integer]: TPrinterInfo read GetPrinter      ;
    property PrinterProperty[Index: Integer]: Boolean      read GetPrinterProp  ;
    property PrintersCount                  : Integer      read GetPrintersCount;

    property Job            [Index: Integer]: TJobInfo     read GetJobInfo      ;
    property JobsCount                      : Integer      read GetJobsCount    ;

  published
    { Published declarations }
  end;

implementation

constructor TPrinterEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPrinters := TList.Create;
  FJobs     := TList.Create;
  FPrinters.Capacity := 30;
  FJobs    .Capacity := 30;

  UpdatePrinters;
  UpdatePrinters;
end;

destructor TPrinterEx.Destroy;
Var
  I: Integer;
begin
  for I := 0 to FPrinters.Count -1 do Dispose(FPrinters[I]);
  for I := 0 to FJobs    .Count -1 do Dispose(FJobs    [I]);

  FPrinters.Clear;
  FJobs    .Clear;
  FPrinters.Destroy;
  FJobs    .Destroy;
  inherited;
end;

procedure TPrinterEx.UpdatePrinters;
Var
  pcbNeeded  ,
  pcReturned : DWORD;
  Buffer     ,
  PrinterInfo: PChar;
  I          : Integer;
  PPrinter   : PPrinterInfo;
begin

  if FPrinters.Count > 0 then begin
     for I :=0 to FPrinters.Count -1 do Dispose(FPrinters[I]);
     FPrinters.Clear;
  end;

  EnumPrinters(PRINTER_ENUM_LOCAL, nil, 2, nil, 0, pcbNeeded, pcReturned);
  GetMem(Buffer, pcbNeeded);
  EnumPrinters(PRINTER_ENUM_LOCAL, nil, 2, PByte(Buffer), pcbNeeded, pcbNeeded, pcReturned);
  PrinterInfo := Buffer;
  for I := 0 to pcReturned -1 do
    with PPrinterInfo2(PrinterInfo)^ do begin

      New(PPrinter);

      PPrinter^.ServerName     := StrPas(pServerName    );
      PPrinter^.PrinterName    := StrPas(pPrinterName   );
      PPrinter^.ShareName      := StrPas(pShareName     );
      PPrinter^.PortName       := StrPas(pPortName      );
      PPrinter^.DriverName     := StrPas(pDriverName    );
      PPrinter^.Comment        := StrPas(pComment       );
      PPrinter^.Location       := StrPas(pLocation      );
      PPrinter^.SepFile        := StrPas(pSepFile       );
      PPrinter^.PrintProcessor := StrPas(pPrintProcessor);
      PPrinter^.Datatype       := StrPas(pDatatype      );
      PPrinter^.Parameters     := StrPas(pParameters    );
      PPrinter^.Attributes     := Attributes             ;
      PPrinter^.Priority       := Priority               ;
      PPrinter^.DefaultPriority:= DefaultPriority        ;
      PPrinter^.StartTime      := StartTime              ;
      PPrinter^.UntilTime      := UntilTime              ;
      PPrinter^.Status         := Status                 ;
      PPrinter^.Jobs           := cJobs                  ;
      PPrinter^.AveragePPM     := AveragePPM             ;

      FPrinters.Add(Pointer(PPrinter));

      Inc(PrinterInfo, SizeOf(TPrinterInfo2));
    end;
  FreeMem(Buffer);
end;

function TPrinterEx.GetPrinter(Index: Integer): TPrinterInfo;
var
  PPrinter: PPrinterInfo;
begin
  if (Index > FPrinters.Count) and (FPrinters.Count <> 0) then begin
    raise EInvalidIndex.Create(Format(LoadStr(65451), [Index]));
    Exit;
  end;
  if FPrinters.Count <> 0 then begin
    PPrinter := FPrinters.Items[Index];
    with Result do begin
      ServerName     := PPrinter^.ServerName     ;
      PrinterName    := PPrinter^.PrinterName    ;
      ShareName      := PPrinter^.ShareName      ;
      PortName       := PPrinter^.PortName       ;
      DriverName     := PPrinter^.DriverName     ;
      Comment        := PPrinter^.Comment        ;
      Location       := PPrinter^.Location       ;
      SepFile        := PPrinter^.SepFile        ;
      PrintProcessor := PPrinter^.PrintProcessor ;
      Datatype       := PPrinter^.Datatype       ;
      Parameters     := PPrinter^.Parameters     ;
      Attributes     := PPrinter^.Attributes     ;
      Priority       := PPrinter^.Priority       ;
      DefaultPriority:= PPrinter^.DefaultPriority;
      StartTime      := PPrinter^.StartTime      ;
      UntilTime      := PPrinter^.UntilTime      ;
      Status         := PPrinter^.Status         ;
      Jobs           := PPrinter^.Jobs           ;
      AveragePPM     := PPrinter^.AveragePPM     ;
    end;
  end;
end;

function TPrinterEx.GetPrintersCount: Integer;
begin
  Result := FPrinters.Count;
end;

function TPrinterEx.GetPrinterProp(Index: Integer): Boolean;
Var
  hPrinter: THandle;
  PI      : TPrinterInfo;
begin
  Result := False;
  PI     := Self.GetPrinter(Index);
  if OpenPrinter(PChar(PI.PrinterName), hPrinter, nil) then begin
    PrinterProperties((Owner As TForm).Handle, hPrinter);
    ClosePrinter(hPrinter);
    Result := True;
  end;
end;

function TPrinterEx.UpdateJobs(Index: Integer): Boolean;
Var
  hPrinter: THandle;
  PI      : TPrinterInfo;
begin
  Result := False;
  PI     := Self.GetPrinter(Index);
  if OpenPrinter(PChar(PI.PrinterName), hPrinter, nil) then begin
    GetJobs(hPrinter);
    ClosePrinter(hPrinter);
    Result := True;
  end;
end;

procedure TPrinterEx.GetJobs(ID: THandle);
Var
  pcbNeeded ,
  pcReturned: DWORD;
  Buffer    ,
  JobsInfo  : PChar;
  I         : Integer;
  PJobs     : PJobInfo;
begin

  if FJobs.Count > 0 then begin
     for I :=0 to FJobs.Count -1 do Dispose(FJobs[I]);
     FJobs.Clear;
  end;

  EnumJobs(ID, 0, High(Integer), 2, nil, 0, pcbNeeded, pcReturned);
  GetMem(Buffer, pcbNeeded);
  EnumJobs(ID, 0, High(Integer), 2, PByte(Buffer), pcbNeeded, pcbNeeded, pcReturned);
  JobsInfo := Buffer;
  for I := 0 to pcReturned -1 do
    with PJobInfo2(JobsInfo)^ do begin

      New(PJobs);

      PJobs^.JobId          := JobId                  ;
      PJobs^.PrinterName    := StrPas(pPrinterName)   ;
      PJobs^.MachineName    := StrPas(pMachineName)   ;
      PJobs^.UserName       := StrPas(pUserName)      ;
      PJobs^.Document       := StrPas(pDocument)      ;
      PJobs^.NotifyName     := StrPas(pNotifyName)    ;
      PJobs^.Datatype       := StrPas(pDatatype)      ;
      PJobs^.PrintProcessor := StrPas(pPrintProcessor);
      PJobs^.Parameters     := StrPas(pParameters)    ;
      PJobs^.DriverName     := StrPas(pDriverName)    ;
      PJobs^.dStatus        := StrPas(pStatus)        ;
      PJobs^.dwStatus       := Status                 ;
      PJobs^.Priority       := Priority               ;
      PJobs^.Position       := Position               ;
      PJobs^.StartTime      := StartTime              ;
      PJobs^.UntilTime      := UntilTime              ;
      PJobs^.TotalPages     := TotalPages             ;
      PJobs^.Size           := Size                   ;
      PJobs^.Submitted      := Submitted              ;
      PJobs^.Time           := Time                   ;
      PJobs^.PagesPrinted   := PagesPrinted           ;

      FJobs.Add(Pointer(PJobs));

      Inc(JobsInfo, SizeOf(TJobInfo2));
    end;
  FreeMem(Buffer);
end;

function TPrinterEx.GetJobInfo(Index: Integer): TJobInfo;
var
  PJob: PJobInfo;
begin
  if (Index > FPrinters.Count) and (FPrinters.Count <> 0) then begin
    raise EInvalidIndex.Create(Format(LoadStr(65451), [Index]));
    Exit;
  end;
  if FJobs.Count <> 0 then begin
    PJob := FJobs.Items[Index];
    with Result do begin
      JobId          := PJob^.JobId         ;
      PrinterName    := PJob^.PrinterName   ;
      MachineName    := PJob^.MachineName   ;
      UserName       := PJob^.UserName      ;
      Document       := PJob^.Document      ;
      NotifyName     := PJob^.NotifyName    ;
      Datatype       := PJob^.Datatype      ;
      PrintProcessor := PJob^.PrintProcessor;
      Parameters     := PJob^.Parameters    ;
      DriverName     := PJob^.DriverName    ;
      dStatus        := PJob^.dStatus       ;
      dwStatus       := PJob^.dwStatus      ;
      Priority       := PJob^.Priority      ;
      Position       := PJob^.Position      ;
      StartTime      := PJob^.StartTime     ;
      UntilTime      := PJob^.UntilTime     ;
      TotalPages     := PJob^.TotalPages    ;
      Size           := PJob^.Size          ;
      Submitted      := PJob^.Submitted     ;
      Time           := PJob^.Time          ;
      PagesPrinted   := PJob^.PagesPrinted  ;
    end;
  end;
end;

function TPrinterEx.GetJobsCount: Integer;
begin
  Result := FJobs.Count;
end;

end.
