unit RRGMPrintDef;
//
//  Delphi 6 adaptation of the Printer selection component by
//  Herman Steigstra, [e-mail HSteigstra@skzl.nl]
//
//  Following instructions from Jeff Overcash (TeamB) on
//  how to separate design-time and run-time code
//
//  The Fine-Arts Bluesband & Poetry Press - April 2003
//  Richard Minutillo (rgm@fabbnet.net)
//
//  This is the Run-time unit
//
interface

uses
  Classes, Printers, SysUtils, WinSpool, WinProcs, WinTypes,
  StdCtrls, Graphics, Controls, Menus, Messages, Forms;

type
  TPrinterDefinition = class;
  TOrientation = (Portrait,Landscape);
  TDuplexPrinting = (Simplex, LongEdge, ShortEdge);
  TUseWinDefs = (DefaultPrinter, DefaultBin,
   DefaultPaperSize, DefaultDuplex, DefaultPrintQuality);
  TUseWinDefaults = set of TUseWinDefs;
  TPrinterSettings = class(TPersistent)
  private
    FPaperSizename,
    FBinSourceName    : string;
    FCopies           : integer;
    FOrientation      : TOrientation;
    FPrintQuality     : integer;
    FDuplex           : TDuplexPrinting;
    FPaperWidth,
    FPaperLength      : double;
    procedure   SetPaperSizeName(const Name: string);
    function    GetPaperSizeName: string;
    procedure   SetBinSourceName(const Name: string);
    function    GetBinSourceName: string;
    procedure   SetOrientation (Orientation: TOrientation);
    procedure   SetCopies(Value: integer);
    procedure   SetPrintQuality (Quality: integer);
    procedure   SetDuplexPrinting(Value: TDuplexPrinting);
    procedure   SetPaperWidth(Value: double);
    function    GetPaperWidth: double;
    procedure   SetPaperLength(Value: double);
    function    GetPaperLength: double;
    procedure   SetFields;
    function    UserPaperFormat: boolean;
  published
    FPrinterDefinition: TPrinterDefinition;
    property PaperSize: string read GetPaperSizeName write SetPaperSizeName;
    property BinSource: string read GetBinSourceName write SetBinSourceName;
    property Orientation: TOrientation read FOrientation write SetOrientation;
    property Copies: integer read FCopies write SetCopies;
    property PrintQuality: integer read FPrintQuality write  SetPrintQuality;
    property Duplex: TDuplexPrinting read FDuplex write SetDuplexPrinting;
    property PaperWidth: double read GetPaperWidth write SetPaperWidth;
    property PaperLength: double read GetPaperLength write SetPaperLength;
  end;
  TPrinterDefinition = class (TCustomCombobox)
  private
    FDevice,FDriver,
    FPort             : PChar;
    FHandle           : THandle;
{$IFDEF WIN32}
    FDeviceMode       : PDeviceModeA;
{$ELSE}
    FDeviceMode       : PDevMode;
{$ENDIF}
    FPrinterName      : string;
    FProfile          : string[80];
    FJobCountChange,
    FPrinterChange,
    FPaperChange,
    FBinChange,
    FOrientationChange: TNotifyEvent;
    FJobCount,
    FBin,FPaperSize   : integer;
    FPrinterChanging  : boolean;
    FComboPaperSize,
    FComboPaperSource : TComboBox;
    FPrinter          : TPrinterSettings;
    FWindowHandle     : HWnd;
    FNewWndProcPtr,
    FOldWndProcPtr    : TFarProc;
    FUseWinDefaults   : TUseWinDefaults;
    procedure   SetPaper_BinArray;
    function    GetDriverName: string;
    function    GetDeviceName: string;
    function    GetPortName: string;
    procedure   SetPaperSizeName(Name: string);
    function    GetPaperSizeName: string;
    procedure   SetBinSourceName(Name: string);
    function    GetBinSourceName: string;
    procedure   SetPrinterName(const Name: string);
    function    GetPrinterName: string;
    function    GetSection: string;
    procedure   SetSection(const Name: string);
    procedure   SetComboPapersize(Box: TComboBox);
    procedure   SetUseWinDefaults(const UseWinDefs: TUseWinDefaults);
    procedure   Loaded; override;
    procedure   SetDeviceMode;
    function    GetHandle: boolean;
    procedure   CloseHandle;
    procedure   PrinterChanged;
    procedure   ComboChanged(Sender: TObject);
    procedure   BinChanged(PrinterChanged: boolean);
    procedure   PaperChanged(PrinterChanged: boolean);
    procedure   WndProcLocal(var Msg: TMessage);
    procedure   ReplaceWndProc;
    procedure   Notification(AComponent: TComponent;
                             Operation: TOperation); override;
  public
    FPrinterList,
    FBinList,
    FPaperList        : TStringList;
    FNumResolutions   : integer;
    ResolutionArray: array[0..25] of
      record
        XResol,YResol: longint;
      end;
    constructor Create (AOwner: TComponent); override;
    destructor  Destroy;  override;
    property    Printers: TStringList read FPrinterList;
    property    PaperSizes: TStringList read FPaperList;
    property    BinSources: TStringList read FBinList;
    property    Driver: string read GetDriverName;
    property    Device: string read GetDeviceName;
    property    Port: string read GetPortName;
    property    JobCount: integer read FJobCount;
    property    Text;

    procedure   StoreProfile;
    procedure   ReadProfile;
    procedure   SaveToDefaults;
    procedure   SavePrinterAsDefault;
    function    UserPaperFormat: boolean;
    procedure   CheckSpoolCount(MaxJobs: integer);
    procedure   RestorePrinterSettings;
  published
    property Style;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;

    property PrinterName: string read GetPrinterName write SetPrinterName;
    property PrinterSettings: TPrinterSettings read FPrinter write FPrinter;
    property Profile: string read GetSection write SetSection;
    property UseWinDefaults: TUseWinDefaults read FUseWinDefaults write SetUseWinDefaults;

    property ComboPaperSize: TComboBox read FComboPapersize write SetComboPapersize;
    property ComboPaperSource: TComboBox read FComboPaperSource write FComboPaperSource;
    property OnPrinterChange: TNotifyEvent read FPrinterChange write FPrinterChange;
    property OnPaperChange: TNotifyEvent read FPaperChange write FPaperChange;
    property OnBinChange: TNotifyEvent read FBinChange write FBinChange;
    property OnOrientationChange: TNotifyEvent read FOrientationChange write FOrientationChange;
    property OnJobCountChange: TNotifyEvent read FJobCountChange write FJobCountChange;
  end;

const Draft = -1;
      Low   = -2;
      Medium = -3;
      High = -4;

{$IFDEF Win32}
function DeviceCapabilitiesA(pDevice, pPort: PAnsiChar; fwCapability: Word;
  pOutput: PAnsiChar; DevMode: PDeviceModeA): Integer; stdcall;
function DeviceCapabilitiesW(pDevice, pPort: PWideChar; fwCapability: Word;
  pOutput: PWideChar; DevMode: PDeviceModeW): Integer; stdcall;
function DeviceCapabilities(pDevice, pPort: PChar; fwCapability: Word;
  pOutput: PChar; DevMode: PDeviceMode): Integer; stdcall;
{$ENDIF}

implementation
{ $I PrintSel.inc}
const
  MaxBins = 25;
  MaxPaperSizes = 100;
  CPaperName = 64;

type
  TWordRef = class(TObject)
    Index: word;
    PaperMetrics: TPoint;
  end;

{$IFDEF Win32}
function DeviceCapabilitiesA; external winspl name 'DeviceCapabilitiesA';
function DeviceCapabilitiesW; external winspl name 'DeviceCapabilitiesW';
function DeviceCapabilities; external winspl name 'DeviceCapabilitiesA';
{$ENDIF}
{ ************************************************************* }

constructor TPrinterDefinition.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  GetMem (FDevice, 255);
  GetMem (FDriver, 255);
  GetMem (FPort, 255);
  Style:=csDropDownList;
  OnChange:=ComboChanged;
  FPrinterChanging:=true;
  FPrinter:=TPrinterSettings.Create;
  FPrinter.FPrinterDefinition:=self;
  FPrinter.FCopies:=1;
  FPrinter.FPrintQuality:=-4;
  FPaperList:=TStringList.Create;
  FBinList:=TStringList.Create;
  FPrinterList:=TStringList.Create;
  FPrinterList.Assign(Printer.Printers);
  SetDeviceMode;
  if not (csDesigning in ComponentState) then ReplaceWndProc;
{$IFDEF NotRegistered}
  CheckMessage(self);
{$ENDIF}
end;

destructor TPrinterDefinition.Destroy;
begin
  if FNewWndProcPtr <> nil then
    begin
      SetWindowLong(FWindowHandle,gwl_wndProc, longint(FOldWndProcPtr));
      FreeObjectInstance(FNewWndProcPtr);
    end;
  if FDevice <> nil then FreeMem (FDevice, 255);
  if FDriver <> nil then FreeMem (FDriver, 255);
  if FPort <> nil then   FreeMem (FPort, 255);
  FPaperList.Free;
  FBinList.Free;
  FPrinterList.Free;
  FPrinter.Free;
  inherited Destroy;
end;

procedure TPrinterDefinition.Loaded;
var Bin,Pap: string;
begin
  inherited Loaded;
  Bin:=FPrinter.FBinSourceName;
  Pap:=FPrinter.FPaperSizeName;
  FPrinterChanging:=false;
  PrinterChanged;
  FPrinter.PaperSize:=Pap;
  FPrinter.BinSource:=Bin;
end;

procedure TPrinterDefinition.SetDeviceMode;
{$IFNDEF WIN32}
var
  Handle:   THandle;
  ExtDevCaps:  TFarProc;
{$ENDIF}
begin
  try
    if GetHandle then
    begin
      CloseHandle;
{$IFNDEF WIN32}
      Handle := LoadLibrary (FDriver);
      if Handle <> 0 then
        begin
          ExtDevCaps := GetProcAddress (Handle, 'ExtDeviceMode');
          if ExtDevCaps<>nil then
            begin
              fillchar(FDeviceMode^,sizeof(FDeviceMode^),0);
              FDeviceMode^.dmFields:=0;
              if TExtDeviceMode (ExtDevCaps)(0, Handle, FDeviceMode^,
               FDevice, FPort, FDeviceMode^, nil,
               DM_IN_BUFFER or DM_OUT_BUFFER)<> IDOK then
                raise EPrinter.Create ('Error updating printer driver.');
            end;
          FreeLibrary (Handle);
        end
      else
        Raise EPrinter.Create ('Error loading driver '+StrPas (FDriver));
{$ENDIF}
    end;
  except
  end;
  SetPaper_BinArray;
end;

procedure TPrinterDefinition.SetPaper_BinArray;
type
  TBinName = array[0..23] of Char;
  TBinNameArray   = array[0..MaxBins] of TBinName;
  TcchPaperName = array[0..CPaperName-1] of Char;
  TPaperNames   = array[0..MaxPaperSizes] of TcchPaperName;
var
  BinNameArray:   ^TBinNameArray;
  BinArray:       array[0..MaxBins] of Word;
  SizeArray:      array[0..MaxBins] of TPoint;
  PaperNameArray: ^TPaperNames;
  PaperSizeArray: array[0..MaxPaperSizes] of Word;
  NumPapers,NumBins,
  i,Index         :integer;
  WordPointer:    TWordRef;
{$IFNDEF WIN32}
  DevCaps:        TFarProc;
  Handle:         THandle;
{$ENDIF}
begin
  FBinList.Clear;
  FPaperList.Clear;
{$IFDEF WIN32}
  DeviceCapabilities (FDevice, FPort, DC_Bins,
                        PChar (@BinArray), FDeviceMode);
  new(BinNameArray);
  NumBins := DeviceCapabilities (FDevice, FPort, DC_BinNames,
                           PChar (BinNameArray), FDeviceMode);
  for i := 0 to NumBins-1 do
    begin
      Index:=BinArray[i];
      WordPointer:=TWordRef.Create;
      Wordpointer.Index:=Index;
      FBinList.AddObject (StrPas(BinNameArray^[i]),WordPointer);
    end;
  Dispose(BinNameArray);
  DeviceCapabilities (FDevice, FPort, DC_Papers,
                                   PChar (@PaperSizeArray), FDeviceMode);
  DeviceCapabilities (FDevice, FPort, DC_PaperSize,
                        PChar (@SizeArray), @FDeviceMode);
  new(PaperNameArray);
  NumPapers := DeviceCapabilities (FDevice, FPort, DC_PaperNames,
                                   PChar (PaperNameArray), FDeviceMode);
  for i := 0 to NumPapers-1 do
    begin
      Index:=PaperSizeArray[i];
      WordPointer:=TWordRef.Create;
      WordPointer.Index:=Index;
      WordPointer.PaperMetrics:=SizeArray[i];
      FPaperList.AddObject (StrPas(PaperNameArray^[i]),WordPointer);
    end;
  dispose(PaperNameArray);
  FNumResolutions:=DeviceCapabilities (FDevice, FPort, DC_ENumResolutions,
                     PChar (@ResolutionArray), FDeviceMode);
{$ELSE}  {WIN3x}
  if (FDriver=nil) or (FDeviceMode=nil) then exit;
  BinNameArray := nil;
  Handle := LoadLibrary (FDriver);
  if Handle <> 0 then
  try
    DevCaps := GetProcAddress (Handle, 'DeviceCapabilities');
    if DevCaps<>nil then
    begin
      NumBins := TDeviceCapabilities (DevCaps)(FDevice, FPort, DC_Bins,
                            PChar (@BinArray), FDeviceMode^);
      new(BinNameArray);
      NumBins := TDeviceCapabilities (DevCaps)(FDevice, FPort, DC_BinNames,
                               PChar (BinNameArray), FDeviceMode^);
      for i := 0 to NumBins-1 do
        begin
          Index:=BinArray[i];
          WordPointer:=TWordRef.Create;
          Wordpointer.Index:=Index;
          FBinList.AddObject (StrPas(BinNameArray^[i]),WordPointer);
        end;

      NumPapers := TDeviceCapabilities (DevCaps)(FDevice, FPort, DC_Papers,
                                                    PChar (@PaperSizeArray), FDeviceMode^);
      NumPapers := TDeviceCapabilities (DevCaps)(FDevice, FPort, DC_PaperSize,
                                                    PChar (@SizeArray), FDeviceMode^);
      new(PaperNameArray);
      NumPapers := TDeviceCapabilities (DevCaps)(FDevice, FPort, DC_PaperNames,
                                                   PChar (PaperNameArray), FDeviceMode^);
      for i := 0 to NumPapers-1 do
        begin
          Index:=PaperSizeArray[i];
          WordPointer:=TWordRef.Create;
          Wordpointer.Index:=Index;
          WordPointer.PaperMetrics:=SizeArray[i];
          FPaperList.AddObject (StrPas(PaperNameArray^[i]),WordPointer);
        end;
      dispose(PaperNameArray);
      FNumResolutions:=TDeviceCapabilities (DevCaps)(FDevice, FPort, DC_ENumResolutions,
                        PChar (@ResolutionArray), FDeviceMode^);
    end;
  finally
    if BinNameArray <> nil then Dispose(BinNameArray);
    FreeLibrary (Handle);
  end
  else
    Raise EPrinter.Create ('Error loading driver '+StrPas (FDriver));
{$ENDIF}
end;

function TPrinterDefinition.GetDriverName: string;
begin
  if FDriver<>nil then Result:=StrPas(FDriver)
                  else Result:='';
end;

function TPrinterDefinition.GetDeviceName: string;
begin
  if FDevice<>nil then Result:=StrPas(FDevice)
                  else Result:='';
end;

function TPrinterDefinition.GetPortName: string;
begin
  if FPort<>nil then Result:=StrPas(FPort)
                else Result:='';
end;

function FindNearest(const Name: string; List: TStrings): integer;
var i,j,nCompare,MaxEqual,nDiff: integer;
Label Diff;
begin
  Result:=-1;
  for i:=0 to List.Count-1 do
    if CompareText(List[i],Name)=0 then Result:=i;
  MaxEqual:=0;
  if Result<0 then
    for i:=0 to List.Count-1 do
      begin
        nCompare:=length(List[i]);
        if nCompare>length(Name) then nCompare:=length(Name);
        for j:=1 to nCompare do
          if UpCase(List[i][j])<>UpCase(Name[j])then
            begin
              nDiff:=100*j-abs(ord(UpCase(List[i][j]))-ord(UpCase(Name[j])));
              if nDiff>MaxEqual then
                begin
                  MaxEqual:=nDiff;
                  Result:=i;
                end;
              goto Diff;
            end;
         MaxEqual:=100*nCompare;
         Result:=i;
Diff:
      end;
end;

function TPrinterDefinition.GetHandle: boolean;
begin
  try
    Printer.GetPrinter(FDevice, FDriver, FPort, FHandle);
    if FHandle = 0 then
      begin
        Printer.PrinterIndex := Printer.PrinterIndex;
        Printer.GetPrinter (FDevice, FDriver, FPort, FHandle);
      end;
    if FHandle=0 then Abort;
{$IFDEF WIN32}
    Printer.SetPrinter(FDevice, FDriver, FPort, FHandle);
    FDeviceMode := GlobalLock(FHandle);
{$ELSE}
    Printer.SetPrinter(FDevice, FDriver, FPort, FHandle);
    FDeviceMode:=Ptr(FHandle,0);
{$ENDIF}
    GetHandle:=true;
  except
    GetHandle:=false;
    FDeviceMode:=nil;
  end;
end;

procedure TPrinterDefinition.CloseHandle;
begin
  Printer.SetPrinter (FDevice, FDriver, FPort, FHandle);
{$IFDEF WIN32}
  GlobalUnLock(FHandle);
{$ENDIF}
end;

procedure TPrinterDefinition.SetPaperSizeName(Name: string);
var PaperIndex: integer;
    Changed: boolean;
begin
  if csLoading in ComponentState then FPrinter.FPaperSizename:=Name;
  if FPrinterChanging then exit;
  Changed:=Name<>FPrinter.FPaperSizename;
  if DefaultPaperSize in FUseWinDefaults then Name:=GetPaperSizeName;
  PaperIndex:=FindNearest(Name,FPaperList);
  if PaperIndex>=0 then
    begin
      Name:=FPaperList[PaperIndex];
      if Name<>FPrinter.FPaperSizename then Changed:=true;
      FPaperSize:=TWordRef(FPaperList.Objects[PaperIndex]).Index;
      if not UserPaperFormat then
        begin
          FPrinter.FPaperWidth:=TWordRef(FPaperList.Objects[PaperIndex]).PaperMetrics.x/10;
          FPrinter.FPaperLength:=TWordRef(FPaperList.Objects[PaperIndex]).PaperMetrics.y/10;
        end;
      FPrinter.FPaperSizeName:=Name;
      FPrinter.SetFields;
      if Changed then PaperChanged(false);
    end;
end;

function TPrinterDefinition.GetPaperSizeName: string;
var i: integer;
begin
  Result:=FPrinter.FPaperSizename;
  if GetHandle then
    begin
      for i:=0 to FPaperList.Count-1 do with
        TWordRef(FPaperList.Objects[i]) do
          if Index=FDeviceMode^.dmPaperSize then
             Result:=FPaperList[i];
      CloseHandle;
    end;
end;

procedure TPrinterDefinition.SetBinSourceName(Name: string);
var iBin: integer;
    Changed: boolean;
begin
  if csLoading in ComponentState then FPrinter.FBinSourceName:=Name;
  if FPrinterChanging then exit;
  Changed:=Name<>FPrinter.FBinSourceName;
  if DefaultBin in FUseWinDefaults then Name:=GetBinSourceName;
  iBin:=FindNearest(Name,FBinList);
  if iBin>=0 then
    begin
      Name:=FBinList[iBin];
      if Name<>FPrinter.FBinSourceName then Changed:=true;
      FBin:=TWordRef(FBinList.Objects[iBin]).Index;
      FPrinter.FBinSourceName:=Name;
      FPrinter.SetFields;
      if Changed then BinChanged(false);
    end;
end;

function TPrinterDefinition.GetBinSourceName: string;
var i: integer;
begin
  Result:=FPrinter.FBinSourceName;
  if GetHandle then
    begin
      for i:=0 to FBinList.Count-1 do with
        TWordRef(FBinList.Objects[i]) do
          if Index=FDeviceMode^.dmDefaultSource then Result:=FBinList[i];
      CloseHandle;
    end;
end;

procedure TPrinterDefinition.SetPrinterName(const Name: string);
var Index: integer;
begin
  if DefaultPrinter in FUseWinDefaults then
    Index:=-1
  else
    Index:=FindNearest(Name,Printer.Printers);
  if (Index<>Printer.PrinterIndex) or (FUseWinDefaults<>[]) then
    begin
      Printer.PrinterIndex:=Index;
      if FHandle<>0 then
        begin
          Printer.GetPrinter(FDevice, FDriver, FPort,FHandle);
          FHandle:=0;
          Printer.SetPrinter(FDevice, FDriver, FPort, 0);
        end;
      SetDeviceMode;
      FPrinter.PaperSize:=FPrinter.FPaperSizeName;
      FPrinter.BinSource:=FPrinter.FBinSourceName;
      PrinterChanged;
    end
  else
    SetDeviceMode;
  FPrinterName:=GetPrinterName;
  if not (csLoading in ComponentState) then with FPrinter do SetFields;
end;

function TPrinterDefinition.GetPrinterName: string;
begin
  Result:=Printer.Printers[Printer.PrinterIndex];
end;

function TPrinterDefinition.GetSection: string;
begin
  Result:=FProfile;
end;

procedure TPrinterDefinition.SetSection(const Name: string);
begin
  FProfile:=Name;
end;

procedure TPrinterDefinition.SetUseWinDefaults(const UseWinDefs: TUseWinDefaults);
begin
  FUseWinDefaults:=UseWinDefs;
  RestorePrinterSettings;
end;

procedure TPrinterDefinition.SetComboPapersize(Box: TComboBox);
begin
  FComboPapersize:=Box;
  if Assigned(FComboPapersize) then
    begin
      FComboPapersize.Items.Assign(PaperSizes);
      FComboPapersize.ItemIndex:=FComboPapersize.Items.IndexOf(FPrinter.FPaperSizeName);
    end;
end;

procedure TPrinterDefinition.RestorePrinterSettings;
begin
  PrinterName:=FPrinterName;
  FPrinter.SetFields;
end;

procedure TPrinterDefinition.StoreProfile;
procedure WriteField(const FieldName,Value: string);
var App,Key,Str: array[0..100] of char;
begin
  StrPcopy(App,FProfile);
  StrPcopy(Key,FieldName);
  StrPcopy(Str,Value);
  WriteProfileString(App,Key,Str);
end;

begin
  if FProfile<=' ' then exit;
  WriteField('PrinterName',PrinterName);
  with FPrinter do
    begin
      WriteField('PaperSize',PaperSize);
      WriteField('BinSource',BinSource);
      if Orientation=Portrait then WriteField('Orientation','Portrait')
                              else WriteField('Orientation','LandScape');
      WriteField('PrintQuality',IntToStr(PrintQuality));
      WriteField('Duplex',IntToStr(ord(Duplex)));
    end;
end;

procedure TPrinterDefinition.ReadProfile;
function ReadField(const FieldName,Default: string): string;
var App,Key,Str,TempStr: array[0..100] of char;
begin
  StrPcopy(App,FProfile);
  StrPcopy(Key,FieldName);
  StrPcopy(Str,Default);
{$IFDEF WIN32}
  GetProfileString(App,Key,Str,TempStr,100);
  Result:=StrPas(TempStr);
{$ELSE}
  Result:=GetProfileStr(App,Key,Default);
{$ENDIF}
end;

begin
  if FProfile<=' ' then exit;
  PrinterName:=ReadField('PrinterName',PrinterName);
  with FPrinter do
    begin
      PaperSize:=ReadField('PaperSize',PaperSize);
      BinSource:=ReadField('BinSource',BinSource);
      if copy(ReadField('Orientation','Portrait'),1,1)='L' then
        Orientation:=LandScape
      else
        Orientation:=Portrait;
      PrintQuality:=StrToInt(ReadField('PrintQuality',IntToStr(PrintQuality)));
      Duplex:=TDuplexPrinting(StrToInt(ReadField('Duplex',IntToStr(ord(Duplex)))));
    end;
  PrinterChanged;
end;

procedure TPrinterDefinition.PrinterChanged;
begin
  if FPrinter=nil then exit;
  FPrinterChanging:=true;
  Items.Assign(Printers);
  ItemIndex:=Items.IndexOf(PrinterName);
  BinChanged(true);
  PaperChanged(true);
  FPrinterChanging:=false;
  if Assigned(FPrinterChange) then FPrinterChange(self);
  if Assigned(FPaperChange) then FPaperChange(self);
  if Assigned(FBinChange) then FBinChange(self);
end;

procedure TPrinterDefinition.ComboChanged(Sender: TObject);
begin
  if Text<>PrinterName then PrinterName:=Text;
end;

procedure TPrinterDefinition.SaveToDefaults;
var
  DrvHandle:   THandle;
{$IFNDEF Win32}
  ExtDevCaps:  TFarProc;
{$ENDIF}
procedure ModDefaults;
begin
{$IFDEF Win32}
  if DocumentProperties (0, DrvHandle, FDevice,
       FDeviceMode^, FDeviceMode^,
       DM_IN_BUFFER or DM_UPDATE)=IDOK then
{$ELSE}
  if TExtDeviceMode (ExtDevCaps)
    (0, DrvHandle, FDeviceMode^, FDevice, FPort,
     FDeviceMode^, nil, DM_IN_BUFFER OR DM_UPDATE)=IDOK then
{$ENDIF}
    SendMessage ($FFFF, WM_WININICHANGE, 0, 0);
end;

begin
  GetHandle;
  CloseHandle;
{$IFDEF Win32}
  OpenPrinter(FDevice, DrvHandle, nil);
  ModDefaults;
  ClosePrinter(DrvHandle);
{$ELSE}
  DrvHandle := LoadLibrary (FDriver);
  if DrvHandle<>0 then
    begin
      ExtDevCaps:=GetProcAddress (DrvHandle,'ExtDeviceMode');
      if ExtDevCaps<>nil then ModDefaults;
      FreeLibrary(DrvHandle);
    end;
{$ENDIF}
end;

procedure TPrinterDefinition.SavePrinterAsDefault;
var
  DeviceStr: String;
begin
  DeviceStr := StrPas (FDevice) + ',' + StrPas (FDriver) + ',' + StrPas (FPort) + #0;
  WriteProfileString ('windows', 'device', @DeviceStr[1]);
  WriteProfileString (nil, nil, nil);
  SendMessage ($FFFF, WM_WININICHANGE, 0, 0);
end;

function  TPrinterDefinition.UserPaperFormat: boolean;
begin
  UserPaperFormat:=FPaperSize=DMPAPER_USER;
end;

procedure TPrinterDefinition.CheckSpoolCount(MaxJobs: integer);
begin
  while JobCount>MaxJobs do Application.ProcessMessages;
end;

procedure TPrinterDefinition.BinChanged(PrinterChanged: boolean);
begin
  if Assigned(FComboPaperSource) then
    begin
      if PrinterChanged then FComboPaperSource.Items.Assign(BinSources);
      FComboPaperSource.ItemIndex:=FComboPaperSource.Items.IndexOf(FPrinter.FBinSourceName);
    end;
  if Assigned(FBinChange) then FBinChange(self);
end;

procedure TPrinterDefinition.PaperChanged(PrinterChanged: boolean);
begin
  if Assigned(FComboPapersize) then
    begin
      if PrinterChanged then FComboPapersize.Items.Assign(PaperSizes);
      FComboPapersize.ItemIndex:=FComboPapersize.Items.IndexOf(FPrinter.FPaperSizeName);
    end;
  if Assigned(FPaperChange) then FPaperChange(self);
end;

procedure TPrinterDefinition.WndProcLocal(var Msg: TMessage);
begin
  with Msg do case Msg of
   wm_SpoolerStatus:
     begin
      FJobCount:=lParam;
      if Assigned(FJobCountChange) then FJobCountChange(self);
     end;
   wm_ShowWindow:
     begin
       if wParam<>0 then
         RestorePrinterSettings;
     end;
  end;
  Msg.Result := CallWindowProc(FOldWndProcPtr, FWindowHandle, Msg.Msg, Msg.wParam, Msg.lParam);
end;

procedure TPrinterDefinition.ReplaceWndProc;
begin
  if Owner=nil then exit;
  with (Owner as TForm) do
    begin
      HandleNeeded;
      if FormStyle=fsMDIform then FWindowHandle:=Clienthandle
                             else FWindowHandle:=Handle;
      FOldWndProcPtr:=pointer(GetWindowLong(FWindowHandle,gwl_wndProc));
    end;
  FNewWndProcPtr:=MakeObjectInstance(WndProcLocal);
  if FNewWndProcPtr=nil then
    raise EOutOfResources.Create('Cannot allocate WndProc handle');
  SetWindowLong(FWindowHandle,gwl_wndProc, longint(FNewWndProcPtr));
end;

procedure TPrinterDefinition.Notification(AComponent: TComponent;
            Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation=opRemove then
    begin
      if AComponent=FComboPaperSize then FComboPaperSize:=nil;
      if AComponent=FComboPaperSource then FComboPaperSource:=nil;
    end;
end;
{ *********************** TPrinterSetings ***************** }
procedure TPrinterSettings.SetBinSourceName(const Name: string);
begin
  FPrinterDefinition.SetBinSourceName(Name);
end;

function  TPrinterSettings.GetBinSourceName: string;
begin
  Result:=FPrinterDefinition.GetBinSourceName;
end;

procedure TPrinterSettings.SetPaperSizeName(const Name: string);
begin
  FPrinterDefinition.SetPaperSizeName(Name);
end;

function  TPrinterSettings.GetPaperSizeName: string;
begin
  Result:=FPrinterDefinition.GetPaperSizeName;
end;

procedure TPrinterSettings.SetOrientation (Orientation: TOrientation);
var OChanged: boolean;
begin
  OChanged:=FOrientation<>Orientation;
  FOrientation:=Orientation;
  SetFields;
  with FPrinterDefinition do
    if OChanged and Assigned(FOrientationChange) then FOrientationChange(self);
end;

procedure TPrinterSettings.SetCopies;
begin
  if Value<1 then Value:=1;
  FCopies:=Value;
  SetFields;
end;

procedure TPrinterSettings.SetPrintQuality (Quality: integer);
begin
  FPrintQuality:=Quality;
  SetFields;
end;

procedure TPrinterSettings.SetDuplexPrinting(Value: TDuplexPrinting);
begin
  FDuplex:=Value;
  SetFields;
end;

procedure TPrinterSettings.SetPaperWidth(Value: double);
begin
  if FPrinterDefinition.FDeviceMode^.dmPaperSize=DMPAPER_USER then
     FPaperWidth:=Value;
end;

function TPrinterSettings.GetPaperLength: double;
begin
  Result:=FPaperLength;
end;

procedure TPrinterSettings.SetPaperLength(Value: double);
begin
  if FPrinterDefinition.FDeviceMode^.dmPaperSize=DMPAPER_USER then
     FPaperLength:=Value;
end;

function TPrinterSettings.GetPaperWidth: double;
begin
  Result:=FPaperWidth;
end;

procedure TPrinterSettings.SetFields;
begin
  if not ((csLoading in FPrinterDefinition.ComponentState) or
          (csReading in FPrinterDefinition.ComponentState)) then
  with FPrinterDefinition do if GetHandle then with FDeviceMode^ do
    begin
      dmFields:=DM_PRINTQUALITY or DM_DEFAULTSOURCE or
                DM_PAPERSIZE    or DM_COPIES        or
                DM_ORIENTATION  or DM_DUPLEX        or
                DM_PRINTQUALITY;
      if DefaultPrintQuality in FUseWinDefaults then
        FPrintQuality:=dmPrintQuality
      else
        dmPrintQuality:=FPrintQuality;
      dmCopies:=FCopies;
      if DefaultDuplex in FUseWinDefaults then
        FDuplex:=TDuplexPrinting(dmDuplex-1)
      else
        dmDuplex:=ord(FDuplex)+1;
      if FBin<>dmDefaultSource then
        begin
          if DefaultBin in FUseWinDefaults then
            FBin:=dmDefaultSource
          else
            dmDefaultSource:=FBin;
        end;
      if FPaperSize<>dmPaperSize then
        begin
          if DefaultPaperSize in FUseWinDefaults then
            FPaperSize:=dmPaperSize
          else
            dmPaperSize:=FPaperSize;
        end;
      if UserPaperFormat and not
         (DefaultPaperSize in FUseWinDefaults) then
        begin
          dmPaperWidth:=round(FPaperWidth*10);
          dmPaperLength:=round(FPaperLength*10);
          dmFields := dmFields or DM_PAPERWIDTH or DM_PAPERLENGTH;
        end;
      if Orientation=Portrait then dmOrientation:=dmOrient_Portrait
                              else dmOrientation:=dmOrient_Landscape;
      CloseHandle;
    end;
end;

function  TPrinterSettings.UserPaperFormat: boolean;
begin
  UserpaperFormat:=FPrinterDefinition.UserpaperFormat;
end;

end.

