unit RRGMPrintDefReg;
//
//  Delphi 6 adaptation of the Printer selection component by
//  Herman Steigstra, [e-mail HSteigstra@skzl.nl]
//
//  Following instructions from Jeff Overcash (TeamB) on
//  how to separate design-time and run-time code
//
//  The Fine-Arts Bluesband & Poetry Press - April 2003
//  Richard Minutillo (rgm@fabbnet.net)
//
//  This is the Design-time unit
//
interface

uses  RRGMPrintDef, DesignIntf, DesignEditors, Classes, Printers, WinSpool,
      //Graphics,
      WinProcs, WinTypes, Sysutils;

procedure Register;

implementation

// uses ;

//
//  Design-Time Classes
//

type
  TPaperSizeEdit = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;
  TBinSourceEdit = class(TPaperSizeEdit)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;
  TPrinterEdit = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;
  TPrintQualityEdit = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function  GetValue: string; override;
    procedure SetValue(const value: string); override;
    function  ResolutionName(Value: integer): string;
    function  ResolutionNameI(Index: integer): string;
  end;
  TPrinterDefEdit = class(TClassProperty)
    function GetAttributes: TPropertyAttributes; override;
  end;

//
//  Design-Time Class functions and procedures
//

function TPrinterEdit.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paReadOnly];
end;


procedure TPrinterEdit.GetValues(Proc: TGetStrProc);
var i: integer;
begin
  with TPrinterDefinition(GetComponent(0)) do
    for i:=0 to FPrinterList.Count-1 do Proc(FPrinterList.Strings[i]);
end;

procedure TBinSourceEdit.GetValues(Proc: TGetStrProc);
var i: integer;
begin
  with TPrinterSettings(GetComponent(0)).FPrinterDefinition do
    for i:=0 to FBinList.Count-1 do Proc(FBinList.Strings[i]);
end;

function TPaperSizeEdit.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paReadOnly];
end;


procedure TPaperSizeEdit.GetValues(Proc: TGetStrProc);
var i: integer;
begin
  with TPrinterSettings(GetComponent(0)).FPrinterDefinition do
    for i:=0 to FPaperList.Count-1 do Proc(FPaperList.Strings[i]);
end;

function TPrintQualityEdit.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paReadOnly];
end;

procedure TPrintQualityEdit.GetValues(Proc: TGetStrProc);
var i: integer;
begin
  for i:=-1 downto -4 do Proc(ResolutionName(i));
  with TPrinterSettings(GetComponent(0)).FPrinterDefinition do
    for i:=0 to FNumResolutions-1 do Proc(ResolutionNameI(i));
end;

function  TPrintQualityEdit.GetValue: string;
begin
  GetValue:=ResolutionName(GetOrdValue);
end;

function TPrintQualityEdit.ResolutionName(Value: integer): string;
var i,iBest,Diff,MaxDiff: integer;
begin
  Result:='';
  with TPrinterSettings(GetComponent(0)).FPrinterDefinition do
  case Value of
    -1:Result:='Draft';
    -2:Result:='Low';
    -3:Result:='Medium';
    -4:Result:='High';
1..1200 : begin
          iBest:=-1;
          MaxDiff:=9999;
          for i:=0 to FNumResolutions-1 do
            begin
              Diff:=abs(ResolutionArray[i].XResol-Value);
              if Diff<MaxDiff then
                begin
                  MaxDiff:=Diff;
                  iBest:=i;
                end;
            end;
          if iBest>=0 then Result:=ResolutionNameI(iBest);
        end;
  end;
end;

function TPrintQualityEdit.ResolutionNameI(Index: integer): string;
begin
  with TPrinterSettings(GetComponent(0)).
    FPrinterDefinition.ResolutionArray[Index] do
      Result:=Format('%d x %d dpi',[XResol,YResol]);
end;


procedure TPrintQualityEdit.SetValue(const value: string);
var i: integer;
begin
  with TPrinterSettings(GetComponent(0)).FPrinterDefinition do
    begin
      for i:=-1 downto -4 do if ResolutionName(i)=Value then
        SetOrdValue(i);
      for i:=0 to FNumResolutions-1 do if ResolutionNameI(i)=Value then
        SetOrdValue(ResolutionArray[i].XResol);
    end;
end;

function TPrinterDefEdit.GetAttributes: TPropertyAttributes;
begin
  Result := [paSubProperties, paReadOnly];
end;


//
//  Register Procedure always in Design-Time
//


procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(String),TPrinterSettings,'Binsource',TBinSourceEdit);
  RegisterPropertyEditor(TypeInfo(String),TPrinterSettings,'PaperSize',TPaperSizeEdit);
  RegisterPropertyEditor(TypeInfo(String),TPrinterSettings,'PaperSize',TPaperSizeEdit);
  RegisterPropertyEditor(TypeInfo(String),TPrinterDefinition,'PrinterName',TPrinterEdit);
  RegisterPropertyEditor(TypeInfo(Integer),TPrinterSettings,'PrintQuality',TPrintQualityEdit);
  RegisterPropertyEditor(TypeInfo(TPrinterSettings),TPrinterDefinition,'PrinterSettings',TPrinterDefEdit);
  RegisterComponents('FABPrint', [TPrinterDefinition]);
end;

end.
